/**
 * This interface is needed to bridge between different segment implementations
 * with Soya's ReduxStore implementation.
 *
 * Segment represents a segment of our unified store tree, specifically, a top
 * level key in our state tree:
 *
 * <pre>
 *   state = {
 *     segmentA: {...},
 *     segmentB: [...],
 *     ...
 *   };
 * </pre>
 *
 * IMPORTANT NOTE: All methods described here is considered non public and
 * should be overridden by child class.
 *
 * @CLIENT_SERVER
 */'use strict';Object.defineProperty(exports, '__esModule', { value: true });var _createClass = (function () {function defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ('value' in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}return function (Constructor, protoProps, staticProps) {if (protoProps) defineProperties(Constructor.prototype, protoProps);if (staticProps) defineProperties(Constructor, staticProps);return Constructor;};})();function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError('Cannot call a class as a function');}}var 
Segment = (function () {
  /**
   * @param {Object} config
   * @param {CookieJar} cookieJar
   * @param {Object} dependencyActionCreatorMap
   */
  function Segment(config, cookieJar, dependencyActionCreatorMap) {_classCallCheck(this, Segment);
    this._config = config;
    this._cookieJar = cookieJar;
    this._dependencyActionCreators = dependencyActionCreatorMap;}


  /**
   * Returns an array of Segment classes that this Segment has dependencies to.
   * Child classes can override this static method to declare their Segment
   * dependencies.
   *
   * @returns {Array<Class<Segment>}
   */_createClass(Segment, [{ key: '_generateQueryId', 
























    /**
     * The generated query ID needs to be string since it'll be used by ReduxStore
     * to store query-related data.
     *
     * @param {any} query
     * @return {string}
     */value: 
    function _generateQueryId(query) {}



    /**
     * Uses action creator to create load action of the given query.
     *
     * @param {any} query
     * @param {string} queryId
     * @param {any} segmentState
     * @return {void | Load}
     */ }, { key: '_createLoadFromQuery', value: 
    function _createLoadFromQuery(query, queryId, segmentState) {}



    /**
     * Returns an object that not only return the resulting data, but also whether
     * or not the data is loaded.
     * 
     * @param {any} query
     * @param {string} queryId
     * @param {any} segmentState
     * @return {QueryResult}
     */ }, { key: '_queryState', value: 
    function _queryState(query, queryId, segmentState) {}



    /**
     * Compares two segment states, returns true if the segment state is
     * different.
     *
     * @param {any} segmentStateA
     * @param {any} segmentStateB
     * @return {boolean}
     */ }, { key: '_isStateEqual', value: 
    function _isStateEqual(segmentStateA, segmentStateB) {
      return segmentStateA === segmentStateB;}


    /**
     * Compares pieces of two state. If they are equal return null, otherwise
     * return an array containing the current segment piece.
     *
     * The reason we enclose the piece in array is we need to differentiate
     * between equal values (null) and differing ones (array).
     *
     * @param prevSegmentState
     * @param segmentState
     * @param query
     * @param queryId
     * @return {?Array<any>}
     */ }, { key: '_comparePiece', value: 
    function _comparePiece(prevSegmentState, segmentState, query, queryId) {
      // If state is equal, nothing has changed, since our reducer always
      // re-creates the object.
      if (this._isStateEqual(prevSegmentState, segmentState)) {
        return null;}


      var prevQueryResult = this._queryState(query, queryId, prevSegmentState);
      var queryResult = this._queryState(query, queryId, segmentState);
      if (prevQueryResult.loaded != queryResult.loaded) {
        return [queryResult.data];}


      var prevSegmentPiece = prevQueryResult.data;
      var segmentPiece = queryResult.data;
      if (segmentPiece === prevSegmentPiece) {
        return null;}


      return [segmentPiece];}


    /**
     * Returns a reducer function. Called only once by Store, on registration
     * of a new segment.
     *
     * Reducer is responsible for:
     * 1) Initializing default segment state.
     * 2) Handling changes to segment state.
     *
     * NOTE: We could easily convert this into reduce() method. However in the
     * spirit of redux, reducers should be stateless functions. Let state of
     * subscription, queries, etc. be handled by others. We're making it
     * much simpler this way, methinks.
     *
     * @return {Function}
     */ }, { key: '_getReducer', value: 
    function _getReducer() {}



    /**
     * Returns an object containing action functions. Unlike reducer,
     * ActionCreator can be stateful objects. This is allowed since ActionCreator
     * has to deal with caching and AJAX requests.
     *
     * @return {Object}
     */ }, { key: '_getActionCreator', value: 
    function _getActionCreator() {}



    /**
     * Processes the refresh requests according to the given state and generates
     * list of queries that must be run with cache turned off.
     *
     * We need to have this method because we only wanted to refresh states that
     * are already loaded, not the
     *
     * @param {?} segmentState
     * @param {?} refreshRequests
     * @returns {Array<?>}
     */ }, { key: '_processRefreshRequests', value: 
    function _processRefreshRequests(segmentState, refreshRequests) {
      return [];} }], [{ key: 'getSegmentDependencies', value: function getSegmentDependencies() {return [];} /**
                                                                                                               * Get the segment name. Segment's name is hard-coded by each segment
                                                                                                               * implementation and must never change.
                                                                                                               *
                                                                                                               * @return {string}
                                                                                                               */ }, { key: 'id', value: function id() {throw new Error('Segment implementation must provide ID!');} /**
                                                                                                                                                                                                                      * Returns true if this segment requires hydration. Locally stored segments
                                                                                                                                                                                                                      * will probably not require any hydration.
                                                                                                                                                                                                                      *
                                                                                                                                                                                                                      * @return {boolean}
                                                                                                                                                                                                                      */ }, { key: 'shouldHydrate', value: function shouldHydrate() {return true;} }]);return Segment;})();exports['default'] = Segment;module.exports = exports['default'];
//# sourceMappingURL=Segment.js.map